from twisted.protocols.shoutcast import ShoutcastClient
from twisted.internet.protocol import ReconnectingClientFactory
from twisted.internet import reactor

from django.utils.daemonize import become_daemon
from django.utils.encoding import force_unicode
from django.conf import settings

from models import Relay

class StreamMonitor(ShoutcastClient):
    def gotMP3Data(self, data):
        pass

    def gotMetaData(self, metadata):
        title = dict(metadata).get('StreamTitle', "")
        if title:
            title = force_unicode(title, errors='ignore')
            relay = self.factory.relay
            relay.up = True
            relay.save()
            try:
                artist, track = title.split(' - ', 1)
            except ValueError:
                artist = None
                track = None

            try:
                previous = relay.log_set.all()[0].title
            except IndexError:
                previous = None

            if title != previous:
                relay.log_set.create(title=title, artist=artist, track=track)
                print "new track on", relay.pk, (artist, track) if artist and track else title

    def connectionMade(self):
        relay = self.factory.relay
        relay.status = None
        relay.save()
        print "connected to", relay.pk

        self.sendCommand("GET", self.path)
        self.sendHeader("User-Agent", self.userAgent)
        self.sendHeader("Icy-MetaData", "1")
        self.endHeaders()

    def connectionLost(self, reason):
        self.handleResponseEnd()

        relay = self.factory.relay
        relay.up = False
        relay.save()
        print "conn lost to", relay.pk

    def handleResponse(self, *args, **kwargs):
        """End of Stream ?"""

class MonitorFactory(ReconnectingClientFactory):
    maxDelay = 60
    def __init__(self, relay):
        self.relay = relay

    def buildProtocol(self, addr):
        p = StreamMonitor("/%s" % str(self.relay.pk))
        p.factory = self
        return p

def connect():
    return [reactor.connectTCP(settings.RELAY_HOST, settings.RELAY_PORT, MonitorFactory(relay))
            for relay in Relay.objects.all()]

def start(**options):
    if options['detach']:
        become_daemon()
    connect()
    reactor.run()
