from annoying.decorators import render_to, ajax_request
from django.conf import settings
from models import Relay, Log

INDEX_SQL = """
SELECT
    id,
    relay_id,
    relay.title AS station, relay.site AS site,
    artist, track, log.title AS title
FROM radiosite_log log
LEFT JOIN conf_relay relay ON relay.local_mount = relay_id
WHERE id IN (select MAX(id) FROM radiosite_log GROUP BY relay_id)
ORDER BY relay_id;
"""

@render_to('index.html')
def index(request):
    
    return {'index': Log.objects.raw(INDEX_SQL),
            'RELAY_BASE': settings.RELAY_BASE}

@ajax_request
def status(request):
    data = {}
    for relay in Relay.objects.all():
        stats = {'up': relay.up}
        try:
            log = relay.log_set.all()[0] 
            stats['title'] = log.title
            stats['artist'] = log.artist
            stats['track'] = log.track
        except IndexError:
            pass
        data[relay.pk] = stats
    return data

@render_to('station.html')
def station(request, pk):
    relay = Relay.objects.get(pk=pk)
    if 'inner' in request.GET:
        log = relay.log_set.values()[:20]
        return {'log': log, 'TEMPLATE': 'station_inner.html'}
    return {'relay': relay, 'RELAY_BASE': settings.RELAY_BASE}

@render_to('bookmarks.xml')
def bookmarks(request):
    return {'objects': Relay.objects.all(), 'RELAY_BASE': settings.RELAY_BASE}
